import json
import time
from json import JSONDecodeError
from urllib import request, parse


class ApiError(Exception):
    def __init__(self, error_code, error_msg):
        self.error_code = error_code
        self.error_msg = error_msg

    def __str__(self):
        return f"ErrorCode: {self.error_code}, ErrorMsg: {self.error_msg}"


class ApiResponse:
    def __init__(self, api: "BaseApi"):
        self.api = api


class BaseApi:
    base_url = ""
    token_api = "api_gateway/auth/get_token"

    app_method = "get"
    app_url = ""
    access_key = ""
    access_secret = ""

    def __init__(self):
        self.token_str = ""
        self.token_expired_time = 0

    @classmethod
    def to_response(cls, response):
        try:
            json_response = json.loads(response)
        except JSONDecodeError:
            raise ApiError(error_code=30000, error_msg="json解析错误")
        if json_response.get("error_code") != 0:
            raise ApiError(
                error_code=json_response["error_code"],
                error_msg=json_response.get("error_message"),
            )
        return json_response

    @classmethod
    def get(cls, url, params):
        query_string = parse.urlencode(params)
        u = request.urlopen(f"http://{cls.base_url}/{url}?{query_string}")
        return cls.to_response(u.read())

    @classmethod
    def post(cls, url, params):
        query_string = parse.urlencode(params)
        u = request.urlopen(f"http://{cls.base_url}/{url}", query_string.encode("utf8"))
        return cls.to_response(u.read())

    def refresh_token(self):
        token_response = self.post(
            self.token_api,
            {
                "access_key": self.access_key,
                "access_secret": self.access_secret,
            },
        ).get('data')
        self.token_str = token_response.get("token")
        self.token_expired_time = token_response.get("expire_time")

    def _api_response(self, token_update=True, **kwargs):
        app_url = kwargs.get("app_url", self.app_url)
        app_method = kwargs.get("api_method", self.app_method)
        if not self.token_str or time.time() > self.token_expired_time:
            self.refresh_token()
        try:
            kwargs["token"] = self.token_str
            if app_method.lower() == "get":
                json_response = self.get(app_url, kwargs)
            elif app_method.lower() == "post":
                json_response = self.post(app_url, kwargs)
            else:
                raise ApiError(error_code=30001, error_msg="目前仅支持post和get的接口")
        except ApiError as e:
            if token_update and e.error_code == 40002:
                return self._api_response(False, **kwargs)
            else:
                raise e
        return json_response

    def api_response(self, **kwargs):
        return self._api_response(**kwargs)



import json
import re
from functools import partial
from airworks_api.base_api import BaseApi, ApiError


class AirWorksApi:

    def __init__(self, base_url, access_key, access_secret, default_app_id=None, **kwargs):
        self.base_api = BaseApi()
        BaseApi.base_url = base_url
        self.base_api.access_key = access_key
        self.base_api.access_secret = access_secret
        self.default_app_id = default_app_id
        self.default_api_method = kwargs.get("default_api_method", "GET")
        self.default_page_num = kwargs.get("default_page_num", 1)
        self.default_page_size = kwargs.get("default_page_size", 100)
        self.app_url = kwargs.get("app_url")
        self.app_method = kwargs.get("app_method", "GET")

    def _call(self, api_url, **kwargs):
        app_id = kwargs.get("app_id", self.default_app_id)
        if not app_id:
            raise ApiError(
                error_code=40009,
                error_msg="app_Id缺失，请先设置default_app_id或传入app_id"
            )
        api_url = self._url_transfer(api_url)

        kwargs.update({
            "app_url": f"api_gateway/api/{app_id}/{api_url}",
            "api_method": kwargs.get("api_method") or self.default_api_method,
            "page_num": kwargs.get("page_num") or self.default_page_num,
            "page_size": kwargs.get("page_size") or self.default_page_size,
        })
        return self.base_api.api_response(**kwargs)

    @classmethod
    def _url_transfer(cls, url):
        url_list = list()
        for index, item in enumerate(url.split("__")):
            if index == 0 and re.match(r"_[0-9]", item):
                item = item[1:]
            url_list.append(item.replace("_A", "/"))
        url = "_".join(url_list)
        return url

    def call(self, **kwargs):
        return self.base_api.api_response(**kwargs)

    def __getattr__(self, key):
        return partial(self._call, api_url=key)


if __name__ == "__main__":
    awp = AirWorksApi(
        base_url="aw-airworks-frontend:30000",
        access_key="kB85aqPMFZs_14",
        access_secret="lh66YHfiE7qL6TcOOvbLTg",
        default_app_id=1
    )

    res = awp.baymax(
        api_method="get",
        country="中国",
        id=None,
        page_size=100,
        page_num=1)

    print(json.dumps(res, indent=2, ensure_ascii=False))



import json

from airworks_api.base_api import BaseApi


class CommonApi(BaseApi):
    base_url = "aw-airworks-frontend:30000"

    app_method = "GET"
    app_url = "api_gateway/api/1/baymax"
    access_key = "kB85aqPMFZs_14"
    access_secret = "lh66YHfiE7qL6TcOOvbLTg"
    query_builder = """"""

    def api_response(
        self,
        country,
        id,
        page_size=100,
        page_num=1,
    ):
        return super(CommonApi, self).api_response(
            country=country,
            id=id,
            page_size=page_size,
            page_num=page_num,
            __query_builder__=self.query_builder,
        )


if __name__ == "__main__":
    api_result = CommonApi().api_response(
        country='中国',
        id=None,
        page_size=10,
    )
    print(json.dumps(api_result, indent=2, ensure_ascii=False))



# -*- coding: utf-8 -*
import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="airworks-api",
    version="0.0.8",
    scripts=[],
    author="valuesimplex",
    description="Tools for airworks airworks_api sdk",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/coder-baymax/airworks",
    packages=setuptools.find_packages(),
    python_requires=">=3.5",
    classifiers=[
        # 发展时期,常见的如下
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        # 开发的目标用户
        'Intended Audience :: Developers',
        # 属于什么类型
        'Topic :: Software Development :: Build Tools',
        # 许可证信息
        'License :: OSI Approved :: MIT License',
        # 目标 Python 版本
        'Programming Language :: Python :: 3',
    ],
)
